==> dabs-0.4.tar.bz2: dabs-0.4/dabs.c <==
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <complex.h>

#include "phy/signal.h"
#include "utils/file.h"
#include "phy/frequency_offset.h"
#include "utils/fft.h"
#include "phy/channel.h"
#include "phy/qpsk.h"
#include "transport/fic.h"
#include "transport/fig.h"
#include "coding/frequency_interleaving.h"
#include "transport/msc.h"
#include "sdr/sdr.h"
#include "utils/io.h"
#include "audio/alsa.h"
#include "utils/log.h"

int time_sync(io *in)
{
  complex ts[T_F + T_NULL];
  int c;
  double energy;
  double min_energy;
  int min_pos;
  int i;

  c = in->get_samples(in, ts, T_F + T_NULL);
  if (c != T_F + T_NULL)
    return -1;

  energy = 0;

  for (i = 0; i < T_NULL; i++)
    energy += cabs(ts[i]);

  min_pos = 0;
  min_energy = energy;

  for (i = T_NULL; i < T_F + T_NULL; i++) {
    energy -= cabs(ts[i - T_NULL]);
    energy += cabs(ts[i]);
    if (energy < min_energy) {
      min_energy = energy;
      min_pos = i - T_NULL + 1;
    }
  }

  if (min_pos > T_F)
    min_pos -= T_F;

  in->put_back_samples(in, &ts[min_pos], T_F + T_NULL - min_pos);

  return min_pos;
}

int fast_time_sync(io *in)
{
  complex ts[T_NULL + 200];
  int c;
  double energy;
  double min_energy;
  int min_pos;
  int i;

  c = in->get_samples(in, ts, T_NULL + 200);
  if (c != T_NULL + 200)
    return -1;

  energy = 0;

  for (i = 0; i < T_NULL; i++)
    energy += cabs(ts[i]);

  min_pos = 0;
  min_energy = energy;

  for (i = T_NULL; i < T_NULL + 200; i++) {
    energy -= cabs(ts[i - T_NULL]);
    energy += cabs(ts[i]);
    if (energy < min_energy) {
      min_energy = energy;
      min_pos = i - T_NULL + 1;
    }
  }

  if (min_pos > 200)
    min_pos -= 200;

  in->put_back_samples(in, &ts[min_pos], T_NULL + 200 - min_pos);

  return min_pos;
}

#include <sys/select.h>

int zap_channel(double *freq, int *subc)
{
  struct {
    double freq;
    int subchannel;
  } cs[2] = {
    { 225648000, 1 },
    { 216928000, 2 }
  };
  static int cur_chan = 0;

  char c;
  fd_set r;
  int ret = 0;
  int s;
  struct timeval tout;
next:
  tout.tv_sec = tout.tv_usec = 0;
  FD_ZERO(&r);
  FD_SET(0, &r);
  s = select(1, &r, NULL, NULL, &tout);
  if (s == -1) { error("select failed\n"); exit(1); }
  if (!FD_ISSET(0, &r)) return ret;
  if (scanf("%c", &c) != 1) return ret;
  if (c == 'n') {
    cur_chan++;
    if (cur_chan > 1) cur_chan = 0;
    *subc = cs[cur_chan].subchannel;
    *freq = cs[cur_chan].freq;
    ret = 1;
  }
  if (c == 'p') {
    cur_chan--;
    if (cur_chan < 0) cur_chan = 1;
    *subc = cs[cur_chan].subchannel;
    *freq = cs[cur_chan].freq;
    ret = 1;
  }
  goto next;
}

void usage(void)
{
  printf("usage: [options]\n");
  printf("options:\n");
  printf("    -f <file>\n");
  printf("        load samples from file\n");
  printf("    -loop\n");
  printf("        loop input file\n");
  printf("    -d\n");
  printf("        debug logs\n");
  printf("    -w\n");
  printf("        warning logs\n");
  exit(0);
}

int main(int n, char **v)
{
  char *in_name = NULL;
  io *in;
  int i;
  int j;
  int start;
  double fine_freq_offset;
  complex *in_time;
  complex *_cur_freq;
  complex *_prev_freq;
  complex *cur_freq;
  complex *prev_freq;
  complex *channel;
  complex *z_ref;
  fft_data *fft_data;
  int freq_offset_cur_pos;
  complex qpsk[K];
  int p[K*2];
  fic *fic;
  frequency_interleaver *fl;
  frame_figs figs;
  int frame_cif_id;
  alsa_out *alsa;
  msc_config *msc_config;
  int fig0_0_errors;
  int do_loop = 0;

  for (i = 1; i < n; i++) {
    if (!strcmp(v[i], "-loop")) { do_loop = 1; continue; }
    if (!strcmp(v[i], "-f")) {if(i>n-2)usage(); in_name = v[++i]; continue; }
    if (!strcmp(v[i], "-d")) { log_flags |= DEBUG; continue; }
    if (!strcmp(v[i], "-w")) { log_flags |= WARN; continue; }
    usage();
  }

  double start_frequency;
  int start_subchannel;
  start_frequency = 225648000;
  start_subchannel = 1;

  fic = new_fic();
  alsa = new_alsa_out();
  msc_config = new_msc_config(alsa->receive_audio, alsa);

  msc_set_channel(msc_config, start_frequency, start_subchannel);

  fl = new_frequency_interleaver();

  init_frequency_offset();

  in_time = malloc(sizeof(complex) * T_S);
  _cur_freq = malloc(sizeof(complex) * T_U);
  _prev_freq = malloc(sizeof(complex) * T_U);
  channel = malloc(sizeof(complex) * T_U);
  z_ref = malloc(sizeof(complex) * T_U);
  if (in_time == NULL || _cur_freq == NULL || _prev_freq == NULL
      || channel == NULL || z_ref == NULL) abort();

  cur_freq = _cur_freq;
  prev_freq = _prev_freq;

  fft_data = init_fft(T_U);

  compute_phase_ref_full(fft_data, z_ref);

  if (in_name != NULL)
    in = new_input_file(in_name, do_loop);
  else
    in = new_sdr("usrp");
  if (in == NULL) exit(1);

  in->set_gain(in, 1);
  in->set_freq(in, start_frequency); //225648000); //220352000); //201072000); //195936000); //216928000);
  in->start(in);

restart:
  fig0_0_errors = 0;

  start = time_sync(in);
  if (start == -1) goto over;

next_frame:
  ;double freq;
  int subc = start_subchannel;
  if (zap_channel(&freq, &subc)) {
    start_subchannel = subc;
    msc_set_channel(msc_config, freq, subc);
    if (freq != start_frequency) {
      start_frequency = freq;
      in->set_freq(in, start_frequency);
      goto restart;
    }
  }

  debug("#start %d\n", start);

  if (in->get_samples(in, NULL, T_NULL) != T_NULL) goto over;

  figs.size = 0;

  /* symbol 1 */
  if (in->get_samples(in, in_time, T_S) != T_S) goto over;
  fine_freq_offset = frequency_offset_fine(in_time);
  debug("# %g ", fine_freq_offset);
  freq_offset_cur_pos = remove_frequency_offset(0, 
                            in_time, fine_freq_offset);
  fine_freq_offset = frequency_offset_fine(in_time);
  debug("%g\n", fine_freq_offset);

  fft(fft_data, &in_time[Delta], prev_freq);

  //channel_estimate(z_ref, prev_freq, channel);
  //channel_compensate(prev_freq, channel, prev_freq);

  for (i = 2; i < 77; i++) {
    if (in->get_samples(in, in_time, T_S) != T_S) goto over;
    fine_freq_offset = frequency_offset_fine(in_time);
    debug("#%g ", fine_freq_offset);
    freq_offset_cur_pos = remove_frequency_offset(freq_offset_cur_pos,
                              in_time, fine_freq_offset);
    fine_freq_offset = frequency_offset_fine(in_time);
    debug("%g\n", fine_freq_offset);

    fft(fft_data, &in_time[Delta], cur_freq);

    //channel_compensate(cur_freq, channel, cur_freq);

    /* frequency de-interleaving done here (using k2n) */
    /* positive frequencies [1..768] */
    for (j = 0; j < K/2; j++)
      qpsk[fl->k2n[FK(j+1)]] = cur_freq[j+1] * conj(prev_freq[j+1]);
    /* negative frequencies [-768..-1] */
    for (j = 0; j < K/2; j++)
      qpsk[fl->k2n[FK(j-K/2)]] = cur_freq[j+T_U-K/2] * conj(prev_freq[j+T_U-K/2]);

    symbol_unmap(p, qpsk);

    /* symbols 2, 3 and 4 are for fic */
    if (i < 5) {
      for (j = 0; j < 4; j++) {
        /* FIB = 256 bits, 30 bytes of data, 2 bytes of crc
         * 4 FIBs per symbol, 12 FIBs for the 3 symbols of the FIC
         * 3 FIBs make for 1 FIC and are used for energy dispersal (768 bits)
         * FIC (768 bits) is convolution encoded, giving 3072+24 bits
         * 3072+24 bits are punctured, giving 2304 bits
         * puncturing is:
         *     21 blocks of 128 bits are punctured by PI=16 (rate 8/24)
         *     (we remove 8 bits over 32)
         *     3 blocks of 128 bits are punctured by PI=15 (rate 9/24)
         *     (we remove 9 bits over 32)
         *     last 24 bits are punctured by 1100 1100 1100 1100 .. 1100
         *     (we remove 12 bits over 24)
         * each symbol gets 3072 bits, so:
         * 1st symbol: fic 0 (2304 bits) + fic 1 (768 bits)
         * 2nd symbol: fic 1 (1536 bits) + fic 2 (1536 bits)
         * 3rd symbol: fic 2 (768 bits)  + fic 3 (2304 bits)
         * we send to the fic unit blocks of 768 bits
         * when the fic unit has enough bits it does the decoding
         */
        receive_fic_bits(&figs, fic, &p[768*j], (i-2)*4+j);
      }
    }

    if (i == 5) {
      /* check that we have FIG 0/0 */
      if (figs.size == 0 || figs.f[0].type != FIG0_0) {
        debug("error; FIG 0/0 not received, skip frame\n");
        fig0_0_errors++;
        if (fig0_0_errors == 5) {
          debug("error: too much missed FIG 0/0, full resync\n");
          goto restart;
        }
        goto frame_over;
      }
      fig0_0_errors = 0;
      frame_cif_id = figs.f[0].u.fig0_0.cif_count;
      /* get sub channels configuration */
      msc_update_config(msc_config, &figs);
    }

    /* symbols 5..76 are for MSC */
    if (i >= 5)
      receive_msc_bits(msc_config, frame_cif_id + (i-5)/18, p, (i-5) % 18);

#if 0
{
  int i;

static int tt = 0;
if (tt == 2500)
  for (i = 0; i < K; i++) debug("%g %g\n", creal(qpsk[i]), cimag(qpsk[i]));
if (tt == 2500)
exit(0);
tt++;
}
#endif

#define  SWAP(a, b) do { complex *x = a; a = b; b = x; } while (0)
    SWAP(cur_freq, prev_freq);
  }

frame_over:
  /* we may have to skip some symbols in case of error */
  for (; i < 77; i++) {
    if (in->get_samples(in, in_time, T_S) != T_S) goto over;
  }

  /* put back last 100 samples of last symbol, for fast time sync */
  in->put_back_samples(in, &in_time[T_S-100], 100);
  start = fast_time_sync(in);
  if (start == -1) goto restart;

  goto next_frame;

over:
  in->stop(in);
  in->free(in);

  return 0;
}

==> dabs-0.5.tar.bz2: dabs-0.5/dabs.c <==
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <complex.h>

#include "phy/signal.h"
#include "utils/file.h"
#include "phy/frequency_offset.h"
#include "utils/fft.h"
#include "phy/channel.h"
#include "phy/qpsk.h"
#include "transport/fic.h"
#include "transport/fig.h"
#include "coding/frequency_interleaving.h"
#include "transport/msc.h"
#include "sdr/sdr.h"
#include "utils/io.h"
#include "audio/alsa.h"
#include "utils/log.h"
#include "utils/scan.h"
#include "utils/database.h"
#include "utils/band3.h"

int time_sync(io *in)
{
  complex ts[T_F + T_NULL];
  int c;
  double energy;
  double min_energy;
  int min_pos;
  int i;

  c = in->get_samples(in, ts, T_F + T_NULL);
  if (c != T_F + T_NULL)
    return -1;

  energy = 0;

  for (i = 0; i < T_NULL; i++)
    energy += cabs(ts[i]);

  min_pos = 0;
  min_energy = energy;

  for (i = T_NULL; i < T_F + T_NULL; i++) {
    energy -= cabs(ts[i - T_NULL]);
    energy += cabs(ts[i]);
    if (energy < min_energy) {
      min_energy = energy;
      min_pos = i - T_NULL + 1;
    }
  }

  if (min_pos > T_F)
    min_pos -= T_F;

  in->put_back_samples(in, &ts[min_pos], T_F + T_NULL - min_pos);

  return min_pos;
}

int fast_time_sync(io *in)
{
  complex ts[T_NULL + 200];
  int c;
  double energy;
  double min_energy;
  int min_pos;
  int i;

  c = in->get_samples(in, ts, T_NULL + 200);
  if (c != T_NULL + 200)
    return -1;

  energy = 0;

  for (i = 0; i < T_NULL; i++)
    energy += cabs(ts[i]);

  min_pos = 0;
  min_energy = energy;

  for (i = T_NULL; i < T_NULL + 200; i++) {
    energy -= cabs(ts[i - T_NULL]);
    energy += cabs(ts[i]);
    if (energy < min_energy) {
      min_energy = energy;
      min_pos = i - T_NULL + 1;
    }
  }

  if (min_pos > 200)
    min_pos -= 200;

  in->put_back_samples(in, &ts[min_pos], T_NULL + 200 - min_pos);

  return min_pos;
}

#include <sys/select.h>

int zap_channel(database_t *d, int *current_radio)
{
  char c;
  fd_set r;
  int ret = 0;
  int s;
  struct timeval tout;
next:
  tout.tv_sec = tout.tv_usec = 0;
  FD_ZERO(&r);
  FD_SET(0, &r);
  s = select(1, &r, NULL, NULL, &tout);
  if (s == -1) { error("select failed\n"); exit(1); }
  if (!FD_ISSET(0, &r)) return ret;
  if (scanf("%c", &c) != 1) return ret;
  if (c == 'n') {
    (*current_radio)++;
    if (*current_radio > d->count) *current_radio = 0;
    ret = 1;
  }
  if (c == 'p') {
    (*current_radio)--;
    if (*current_radio < 0) *current_radio = d->count - 1;
    ret = 1;
  }
  goto next;
}

void usage(void)
{
  printf("usage: [options] <channels file>\n");
  printf("options:\n");
  printf("    -f <file>\n");
  printf("        load samples from file\n");
  printf("    -loop\n");
  printf("        loop input file\n");
  printf("    -d\n");
  printf("        debug logs\n");
  printf("    -w\n");
  printf("        warning logs\n");
  printf("    -scan\n");
  printf("        scan channels of DAB band 3 then exit\n");
  exit(0);
}

int main(int n, char **v)
{
  char *channels_file = NULL;
  char *in_name = NULL;
  io *in;
  int i;
  int j;
  int start;
  double fine_freq_offset;
  complex *in_time;
  complex *_cur_freq;
  complex *_prev_freq;
  complex *cur_freq;
  complex *prev_freq;
  complex *channel;
  complex *z_ref;
  fft_data *fft_data;
  int freq_offset_cur_pos;
  complex qpsk[K];
  int p[K*2];
  fic *fic;
  frequency_interleaver *fl;
  frame_figs figs;
  int frame_cif_id;
  alsa_out *alsa;
  msc_config *msc_config;
  int fig0_0_errors;
  int do_loop = 0;
  int do_scan = 0;
  database_t d;
  int current_radio = 0;

  for (i = 1; i < n; i++) {
    if (!strcmp(v[i], "-loop")) { do_loop = 1; continue; }
    if (!strcmp(v[i], "-f")) {if(i>n-2)usage(); in_name = v[++i]; continue; }
    if (!strcmp(v[i], "-d")) { log_flags |= DEBUG; continue; }
    if (!strcmp(v[i], "-w")) { log_flags |= WARN; continue; }
    if (!strcmp(v[i], "-scan")) { do_scan = 1; continue; }
    if (channels_file == NULL) { channels_file = v[i]; continue; }
    usage();
  }

  if (do_scan) { scan(); exit(0); }

  if (channels_file == NULL) usage();
  if (init_database(&d, channels_file)) exit(1);
  if (d.count == 0) { error("no channels?\n"); exit(1); }

  double start_frequency;
  int start_subchannel;
  start_frequency = band3[d.c[current_radio].freq_index].freq * 1000000;
  start_subchannel = d.c[current_radio].scid;

  fic = new_fic();
  alsa = new_alsa_out();
  msc_config = new_msc_config(alsa->receive_audio, alsa);

  msc_set_channel(msc_config, start_frequency, start_subchannel);

  fl = new_frequency_interleaver();

  init_frequency_offset();

  in_time = malloc(sizeof(complex) * T_S);
  _cur_freq = malloc(sizeof(complex) * T_U);
  _prev_freq = malloc(sizeof(complex) * T_U);
  channel = malloc(sizeof(complex) * T_U);
  z_ref = malloc(sizeof(complex) * T_U);
  if (in_time == NULL || _cur_freq == NULL || _prev_freq == NULL
      || channel == NULL || z_ref == NULL) abort();

  cur_freq = _cur_freq;
  prev_freq = _prev_freq;

  fft_data = init_fft(T_U);

  compute_phase_ref_full(fft_data, z_ref);

  if (in_name != NULL)
    in = new_input_file(in_name, do_loop);
  else
    in = new_sdr("usrp");
  if (in == NULL) exit(1);

  in->set_gain(in, 1);
  in->set_freq(in, start_frequency);
  in->start(in);

restart:
  fig0_0_errors = 0;

  start = time_sync(in);
  if (start == -1) goto over;

next_frame:
  if (zap_channel(&d, &current_radio)) {
    double freq = band3[d.c[current_radio].freq_index].freq * 1000000;
    start_subchannel = d.c[current_radio].scid;
    info("playing %s\n", d.c[current_radio].name);
    msc_set_channel(msc_config, freq, start_subchannel);
    if (freq != start_frequency) {
      start_frequency = freq;
      in->reset(in);
      in->set_freq(in, start_frequency);
      goto restart;
    }
  }

  debug("#start %d\n", start);

  if (in->get_samples(in, NULL, T_NULL) != T_NULL) goto over;

  figs.size = 0;

  /* symbol 1 */
  if (in->get_samples(in, in_time, T_S) != T_S) goto over;
  fine_freq_offset = frequency_offset_fine(in_time);
  debug("# %g ", fine_freq_offset);
  freq_offset_cur_pos = remove_frequency_offset(0, 
                            in_time, fine_freq_offset);
  fine_freq_offset = frequency_offset_fine(in_time);
  debug("%g\n", fine_freq_offset);

  fft(fft_data, &in_time[Delta], prev_freq);

  //channel_estimate(z_ref, prev_freq, channel);
  //channel_compensate(prev_freq, channel, prev_freq);

  for (i = 2; i < 77; i++) {
    if (in->get_samples(in, in_time, T_S) != T_S) goto over;
    fine_freq_offset = frequency_offset_fine(in_time);
    debug("#%g ", fine_freq_offset);
    freq_offset_cur_pos = remove_frequency_offset(freq_offset_cur_pos,
                              in_time, fine_freq_offset);
    fine_freq_offset = frequency_offset_fine(in_time);
    debug("%g\n", fine_freq_offset);

    fft(fft_data, &in_time[Delta], cur_freq);

    //channel_compensate(cur_freq, channel, cur_freq);

    /* frequency de-interleaving done here (using k2n) */
    /* positive frequencies [1..768] */
    for (j = 0; j < K/2; j++)
      qpsk[fl->k2n[FK(j+1)]] = cur_freq[j+1] * conj(prev_freq[j+1]);
    /* negative frequencies [-768..-1] */
    for (j = 0; j < K/2; j++)
      qpsk[fl->k2n[FK(j-K/2)]] = cur_freq[j+T_U-K/2] * conj(prev_freq[j+T_U-K/2]);

    symbol_unmap(p, qpsk);

    /* symbols 2, 3 and 4 are for fic */
    if (i < 5) {
      for (j = 0; j < 4; j++) {
        /* FIB = 256 bits, 30 bytes of data, 2 bytes of crc
         * 4 FIBs per symbol, 12 FIBs for the 3 symbols of the FIC
         * 3 FIBs make for 1 FIC and are used for energy dispersal (768 bits)
         * FIC (768 bits) is convolution encoded, giving 3072+24 bits
         * 3072+24 bits are punctured, giving 2304 bits
         * puncturing is:
         *     21 blocks of 128 bits are punctured by PI=16 (rate 8/24)
         *     (we remove 8 bits over 32)
         *     3 blocks of 128 bits are punctured by PI=15 (rate 9/24)
         *     (we remove 9 bits over 32)
         *     last 24 bits are punctured by 1100 1100 1100 1100 .. 1100
         *     (we remove 12 bits over 24)
         * each symbol gets 3072 bits, so:
         * 1st symbol: fic 0 (2304 bits) + fic 1 (768 bits)
         * 2nd symbol: fic 1 (1536 bits) + fic 2 (1536 bits)
         * 3rd symbol: fic 2 (768 bits)  + fic 3 (2304 bits)
         * we send to the fic unit blocks of 768 bits
         * when the fic unit has enough bits it does the decoding
         */
        receive_fic_bits(&figs, fic, &p[768*j], (i-2)*4+j);
      }
    }

    if (i == 4) {
      /* check that we have FIG 0/0 */
      if (figs.size == 0 || figs.f[0].type != FIG0_0) {
        debug("error; FIG 0/0 not received, skip frame\n");
        fig0_0_errors++;
        if (fig0_0_errors == 5) {
          debug("error: too much missed FIG 0/0, full resync\n");
          goto restart;
        }
        i++;
        goto frame_over;
      }
      fig0_0_errors = 0;
      frame_cif_id = figs.f[0].u.fig0_0.cif_count;
      /* get sub channels configuration */
      msc_update_config(msc_config, &figs);
    }

    /* symbols 5..76 are for MSC */
    if (i >= 5)
      receive_msc_bits(msc_config, frame_cif_id + (i-5)/18, p, (i-5) % 18);

#if 0
{
  int i;

static int tt = 0;
if (tt == 2500)
  for (i = 0; i < K; i++) debug("%g %g\n", creal(qpsk[i]), cimag(qpsk[i]));
if (tt == 2500)
exit(0);
tt++;
}
#endif

#define  SWAP(a, b) do { complex *x = a; a = b; b = x; } while (0)
    SWAP(cur_freq, prev_freq);
  }

frame_over:
  /* we may have to skip some symbols in case of error */
  for (; i < 77; i++) {
    if (in->get_samples(in, in_time, T_S) != T_S) goto over;
  }

  /* put back last 100 samples of last symbol, for fast time sync */
  in->put_back_samples(in, &in_time[T_S-100], 100);
  start = fast_time_sync(in);
  if (start == -1) goto restart;

  goto next_frame;

over:
  in->stop(in);
  in->free(in);

  return 0;
}
